<?php

/* RECORD.PHP - manipulate 'records' and write/read them to/from database

This module "processes" posts as a step between the display code and the 
database: DISPLAY > RECORD > MYSQL. For example:

displayrecord() calls readrecord() which calls dbreadrecord()

dbreadrecord() returns a $record and readrecord() validates and converts the 
record data suitible for displayrecord() to display it via displayhtml()

The following functions are defined:

	readrecord()			// called by DISPLAY and Admin
	writerecord()			// called by Admin
	createcomment()			// creates comment $record from POST
	stringtorecord()		// converts raw post to $record
					//  dbreadrecord() calls this
	recordtostring()		// converts $record to string
	sortrecords()			// sorts list of record ids
	readfilerecord()		// read a "page"
*/


/* readrecord - read an entry (post) by ID and process it into a "record" */

// $data is for admin preview (only ADMIN and DISPLAY call this function)

function readrecord($id, $data = NULL, $headers = FALSE) {

	debug("'$id'");

	if ($data)					// Admin can pass data
		$record = stringtorecord($data);
	else
		$record = dbreadrecord($id);

	if ($record == FALSE) {
		$record = recordcreate(htmlgeterror('NOTPOST'));
		$record['title'] = 'Error';
		$record['nocomments'] = 1;
		return $record;
	}

	$record['id'] = $id;

	recorddefaults($record);			// make sure defaults

	if ($headers) {
		unset($record['body']);
		return $record;
	}

	recordbody($record);				// convert/format

	return $record;
}

/* readfilerecord - read a file in the format of a record */

function readfilerecord($id, $headers = FALSE) {

	$record = dbreadpage($id);
	if ($record == FALSE) {
		$record = recordcreate(htmlgeterror('NOTFOUND'));
		$record['title'] = 'Error';
		return $record;
	}

	recorddefaults($record);

	if ($headers) {
		unset($record['body']);
		return $record;
	}

	recordbody($record);

	return $record;
}

/* writerecord - write a record */

// $headers and $id are Admin thingys

function writerecord($record, $headers = array(), $id = 0) {

	if (!is_array($record))
		$record = stringtorecord($record);

	recorddefaults($record,TRUE);

	if ($headers) {
		foreach ($headers as $k => $v) {
			if (is_numeric($k))
				$record[$v] = '';
			else
				$record[$k] = $v;
		}
	}

	if ($id)
		return dbputrecord($id,$record);

	return dbnewrecord($record);
}


/* readcomment - used by Admin */

function readcomment($id, $data) {

	debug("'$id'");

	if ($data)
		$record = stringtorecord($data);
	else
		$record = dbreadcomment($id);

	$record['id'] = $id;

# not enough data to warrant a full defaulting data array, or is it?
	if (!isset($record['date']))
		$record['date'] = date(config('datestrcmt'));

	if (!isset($record['title']))
		$record['title'] = '';

	return $record;
}


/* stringtorecord - convert record string to associative array */

function stringtorecord($data) {

	$data = explode("\n",$data);

// note 1
# actually, this is ^a very inefficient way to do this^ (it will be fixed)
	foreach ($data as &$d)
		$d .= "\n";
	reset($data);
	unset($d);

	while (list( ,$_) = each($data)) {
		if ($_ == "\n") {
			$_ = '';
			break;
		}

		$h = explode(':',$_);
		if (count($h) == 1)
			break;

		$name = array_shift($h);		//  tag: text
		$name = strtolower($name);		//  tag:
		$value = implode(':',$h);		//  tag: Title:Foobar
		$record[$name] = trim($value);
	}

	$record['body'] = $_;
	while (list( ,$_) = each($data))
		$record['body'] .= $_;

	return $record;
}

/* recordtostring - convert record array into a string */

// the string is "key:value\n\nbody" where there can be zero or more of 
// (basically unknown) key:value\n pairs

function recordtostring($record) {

	$data =  '';
	foreach ($record as $key => $value)
		if ($key != 'body')
			$data .= $key.':'.$value."\n";

	return $data."\n".$record['body'];
}

/* sortrecords - sort the entries (natural sort is the default) */

function _cmptitle($a, $b) { return strcmp($a['title'],$b['title']); }
function _cmptime($a, $b)  { return strcmp($a['time'],$b['time']);   }

function sortrecords(&$ids, $sort = NULL) {

	if ($sort === NULL)
		$sort = config('sortfunc');

	if ($sort == "date") {
		$i = 0;
		foreach ($ids as $id) {
			$headers[$i] = readrecord($id,NULL,TRUE);
			$headers[$i]['id'] = $id;
			$i++;
		}
		usort($headers,'_cmptime');
		$i = 0;
		foreach ($headers as $h) {
			$ids[$i] = $h['id'];
			$i++;
		}
	}
	else
	if ($sort == 'name') {
		$i = 0;
		foreach ($ids as $id) {
			$headers[$i] = readrecord($id,NULL,TRUE);
			$headers[$i]['id'] = $id;
			$i++;
		}
		usort($headers,'_cmptitle');
		$i = 0;
		foreach ($headers as $h) {
			$ids[$i] = $h['id'];
			$i++;
		}
	}

	if (config('reversesort') || $sort == 'rsort')
		$ids = array_reverse($ids);
}

/* recordcreate - create an empty record */

function recordcreate($body = '') {

	$record = array();
	recorddefaults($record,TRUE);
	$record['body'] = $body;
	return $record;
}

/* recorduserdata - add some server data to a record */

function recorduserdata(&$record) {

	if (isset($_SERVER['REMOTE_ADDR']))
		$record['REMOTE_ADDR'] = $_SERVER['REMOTE_ADDR'];
	if (isset($_SERVER['HTTP_USER_AGENT']))
		$record['HTTP_USER_AGENT'] = $_SERVER['HTTP_USER_AGENT'];
	if (isset($_SERVER['HTTP_REFERER']))
		$record['HTTP_REFERER'] = $_SERVER['HTTP_REFERER'];
}

/*
   The rest is for the "default data, filter, stuff", an architecture, if you 
   will, and almost a complete, stand-alone API. The point is is for the code 
   to know nothing about the data, i.e. no if ($record['name']) ... The 
   reasoning is that to add/change record fields you just change the data 
   not the code - there is one required field: 'body'.
*/

define('DEF_DEF','/([a-zA-Z]+)\s+([$]*[a-zA-Z_0-9]+)/'); // data definition
define('DEF_REF','/\{([a-z]+)\}/');			// reference definition

/* recorddefaults - default array members by data definition */

// $record - the array
// $defaults - to just apply defaults, not filters
// $body - apply to "header" fields only

function recorddefaults(&$record, $defaults = FALSE, $body = TRUE) {

	if (config('headers'))
		recorddefaultheaders($record);

	$data = recorddefinition();

	foreach ($data as $name => $def) {
		//echo "$name: ";
		foreach ($def as $key => $val) {
			if ($defaults && $key != 'filter')
				continue;
			_setdefaults($name,$record,$key,$val);
		}
	}
}

/* recordbody - apply stuff to record 'body' */

// $convert is an (optional) function to apply 

function recordbody(&$record, $convert = NULL) {

# an optional header flag; this will be changed to be a define or something
# i.e. the data needs to control this, not the code here (or something else)
	$no = (isset($record['notranslate'])) ? $record['notranslate'] : 0;
	if ($no === 1)
		return;

	// use the passed function, the defined function or our default 

	if (!$convert)
		$convert = (defined('DEF_CONVERT')) ? DEF_CONVERT : 'gmlp_convert';

# and it passes the above flag
	$record['body'] = $convert($record['body'],$no === '');

// a little used, odd thing:
	if ($no === 0)
		recordrefs($record);
}

/* recorddefaultheaders - check for and/or create additional headers */

// a not used yet, aditional way to configure the code

function recorddefaultheaders(&$record) {

	$headers = config('headers');
	if (!is_array($headers))
		return;

	foreach ($headers as $key => $val)
		if (!isset($record[$key]))
			$record[$key] = $val;
}

/* recordrefs - replace record vars (determined by DEF_REF) in record 'body' */

// note 2

function recordrefs(&$record) {

	$re = DEF_REF;
	if (preg_match_all($re,$record['body'],$res)) {
		foreach ($res[0] as $k => $v) {
			$h = $res[1][$k];
			if (isset($record[$h]))
				$record['body'] = str_replace($v,$record[$h],$record['body']);
		}
	}
}

/* data support functions */

function _setdefaults($name, &$record, $key, $val) {

	$arg = '';
	if (preg_match(DEF_DEF,$val,$r)) {
		$val = $r[1];
		$arg = $r[2];
	}
	$f = "_$key";
	$f($val,$record,$name,$arg);
}

function _default($val, &$record, $name) {
	if (isset($record[$name]))
		return;
	if (function_exists($val))
		$val = $val();
	$record[$name] = $val;
}

function _value($val, &$record, $name, $a = '') {
	if (function_exists($val)) {
		if ($a) {
			if ($a[0] == '$')
				$a = $record[substr($a,1)];
			$val = $val($a);
		}
		else
			$val = $val();
	}
	$record[$name] = $val;
}

function _filter($val, &$record, $name, $a = '') {
	if (function_exists($val)) {
		if ($a !== '')
			$val = $val($record[$name],$a);
		else
			$val = $val($record[$name]);
		$record[$name] = $val;
	}
}

/*
   Briefly, array of header fields: 'default' is a string or function and is 
   applied if header is empty; 'value' is always applied (constants); 'filter' 
   is applied to the data. There can be one argument for a function, if '$' it 
   means value of that header field (which makes the fields order dependent).
*/

function recorddefinition() {
return array (
  'date' => 
  array (
    'default' => 'configdate',
  ),
  'time' => 
  array (
    'value' => 'strtotime $date',
  ),
  'datetime' => 
  array (
    'value' => 'configdatetime $time',
  ),
  'title' => 
  array (
    'default' => 'Title',
    'filter' => 'htmlspecialchars '.ENT_QUOTES,
  ),
);
}

# how to make this data configurable via Admin
/*
[date]
default = datestr
[time]
value = strtotime $date
[title]
default = Title
filter = htmlspecialchars ENT_QUOTES
*/

/*
NOTES
1. PREG_SPLIT_DELIM_CAPTURE is wanted here, but this:
	$data = preg_split('/(\n)/',$data,-1,PREG_SPLIT_DELIM_CAPTURE);
puts the "\n" as a separate array member which is not right.
2. This substitutes a [word] in body with a record header value, ie. with a 
record header of "title: Foobar", the string "Hey there, [title]!" would result 
in "Hey there, Foobar!". (Of little use but we think it cool.)
*/
